﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Xml.Serialization;
using Model;

namespace DataAccess
{
    public class DataAccessObjectSerializedFile : IDataAccessObject
    {
        // _books must have a suitable runtime type to make de-serialization work.
        private readonly IList<Book> _books = new Book[] { };
        public const String DefaultFilename = @"..\..\..\books.xml";

        public DataAccessObjectSerializedFile(String filename = DefaultFilename)
        {
            if (string.IsNullOrWhiteSpace(filename))
            {
                throw new ArgumentException("filename is null or empty");
            }
            try
            {
                _books = ReadBooksFromSerializedFile(filename);
            }
            catch (IOException ex)
            {
                throw new LibraryException(ex.Message);
            }
            catch (InvalidOperationException ex)
            {
                throw new LibraryException(ex.Message);
            }
            catch (InvalidCastException ex)
            {
                throw new LibraryException(ex.Message);
            }
        }

        /// <summary>
        /// http://msdn.microsoft.com/en-us/library/ms172872.aspx
        /// </summary>
        /// <param name="filename"></param>
        /// <returns></returns>
        private IList<Book> ReadBooksFromSerializedFile(String filename)
        {
            XmlSerializer serializer = new XmlSerializer(_books.GetType());
            StreamReader file = new StreamReader(filename);
            Object obj = serializer.Deserialize(file);
            IList<Book> books = (IList<Book>)obj;
            // safe case better??
            //IList<Book> books = obj as IList<Book>;
            //if books == null, do what?
            return books;
        }

        public Book GetBookByIsbn(string isbn)
        {
            foreach (Book book in _books)
            {
                if (book.Isbn.Equals(isbn)) return book;
            }
            return null;
        }

        public IList<Book> GetBookByTitle(string titleFragment)
        {
            IList<Book> result = new List<Book>();
            foreach (Book book in _books)
            {
                if (book.Title.Contains(titleFragment))
                {
                    result.Add(book);
                }
            }
            return result;
        }

        public string Description { get { return "Serialized file"; } }

        /// <summary>
        /// Helper method to get you started: Creates an XML file with a few books
        /// </summary>
        /// <param name="filename"></param>
        static void MakeFile(String filename = DefaultFilename)
        {
            IList<Book> bookList = new Book[]
            {
                new Book("Beginning Android 4 Application Development", "Lee", "478-1-118-19954-1"),
                new Book("Functional Programming in C#", "Sturm", "978-0-470-74458-1"),
                new Book("Head First Design Patterns", "Freeman", "0-596-007-12")
            };
            XmlSerializer serializer = new XmlSerializer(bookList.GetType());
            using (StreamWriter file = new StreamWriter(filename))
            {
                serializer.Serialize(file, bookList);
            }
        }

        public static void Main(string[] args)
        {
            MakeFile();
        }
    }
}
