﻿using System;
using System.Collections.Generic;
using System.IO;
using Model;

namespace DataAccess
{
    public class DataAccessObjectTextFileDirect : IDataAccessObject
    {
        private const string DefaultFileName = @"..\..\..\books.txt";
        private readonly string _filename;

        public DataAccessObjectTextFileDirect(string filename = DefaultFileName)
        {
            if (String.IsNullOrWhiteSpace(filename))
            {
                throw new ArgumentException("filename is null or empty");
            }
            if (!File.Exists(filename))
            {
                throw new LibraryException(string.Format("File does not exist: {0}", filename));
            }
            this._filename = filename;

        }

        public Book GetBookByIsbn(string isbn)
        {
            try
            {
                using (StreamReader reader = new StreamReader(_filename))
                {
                    while (!reader.EndOfStream)
                    {
                        String title = reader.ReadLine();
                        String author = reader.ReadLine();
                        String isbnFromFile = reader.ReadLine();
                        if (isbn.Equals(isbnFromFile))
                        {
                            Book book = new Book(title, author, isbn);
                            return book;
                        }
                    }
                }
                return null;
            }
            catch (IOException ex)
            {
                throw new LibraryException(ex.Message);
            }
        }

        public IList<Book> GetBookByTitle(string titleFragment)
        {
            try
            {
                IList<Book> books = new List<Book>();
                using (StreamReader reader = new StreamReader(_filename))
                {
                    while (!reader.EndOfStream)
                    {
                        String title = reader.ReadLine();
                        String author = reader.ReadLine();
                        String isbn = reader.ReadLine();
                        if (title != null && title.Contains(titleFragment))
                        {
                            Book book = new Book(title, author, isbn);
                            books.Add(book);
                        }
                    }
                }
                return books;
            }
            catch (IOException ex)
            {
                throw new LibraryException(ex.Message);
            }
        }

        public string Description { get { return "Text file Direct"; } }
    }
}
