﻿using System;
using System.Collections.Generic;
using System.IO;
using Model;

namespace DataAccess
{
    public class DataAccessObjectTextFileMemory : IDataAccessObject
    {
        private const string DefaultFileName = @"..\..\..\books.txt";
        private readonly IList<Book> _books;

        public DataAccessObjectTextFileMemory(string filename = DefaultFileName)
        {
            if (String.IsNullOrWhiteSpace(filename))
            {
                throw new ArgumentException("filename is null or empty");
            }
            try
            {
                _books = new List<Book>();
                using (StreamReader reader = new StreamReader(filename))
                {
                    while (!reader.EndOfStream)
                    {
                        String title = reader.ReadLine();
                        String author = reader.ReadLine();
                        String isbn = reader.ReadLine();
                        Book book = new Book(title, author, isbn);
                        _books.Add(book);
                    }
                }
            }
            catch (IOException ex)
            {
                throw new LibraryException("some message", ex);
            }
        }

        public Book GetBookByIsbn(string isbn)
        {
            foreach (Book book in _books)
            {
                if (book.Isbn.Contains(isbn))
                {
                    return book;
                }
            }
            return null;
        }

        public IList<Book> GetBookByTitle(string titleFragment)
        {
            IList<Book> result = new List<Book>();
            foreach (var book in _books)
            {
                if (book.Title.Contains(titleFragment)) result.Add(book);
            }
            return result;
        }

        public string Description { get { return "Text file In Memory"; } }
    }
}
