﻿using System;
using System.Collections.Generic;
using DataAccess;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Model;

namespace UnitTesting
{
    [TestClass]
    public class UnitTestDataAccess
    {
        [TestMethod]
        public void TestDataAccessInMemory()
        {
            IDataAccessObject dao = new DataAccessObjectTextFileMemory();
            IList<Book> books = dao.GetBookByTitle("Android");
            Assert.AreEqual(1, books.Count);
            books = dao.GetBookByTitle("Does not exist");
            Assert.AreEqual(0, books.Count);

            Book book = dao.GetBookByIsbn("1234-5678");
            Assert.IsNull(book);

            book = dao.GetBookByIsbn("978-0-470-74458-1");
            Assert.AreEqual("Sturm", book.Author);
        }

        [TestMethod]
        public void TestDataAccessSerializedFile()
        {
            IDataAccessObject dao = new DataAccessObjectSerializedFile();
            IList<Book> books = dao.GetBookByTitle("Android");
            Console.WriteLine("books: " + books);
            Assert.AreEqual(1, books.Count);
            books = dao.GetBookByTitle("Does not exist");
            Assert.AreEqual(0, books.Count);

            Book book = dao.GetBookByIsbn("1234-5678");
            Assert.IsNull(book);

            book = dao.GetBookByIsbn("978-0-470-74458-1");
            Assert.AreEqual("Sturm", book.Author);

            try
            {
                new DataAccessObjectSerializedFile(@"c:\doesNotExist.txt");
                Assert.Fail();
            }
            catch (LibraryException) { }

            try
            {
                new DataAccessObjectSerializedFile("    ");
                Assert.Fail();
            }
            catch (ArgumentException ex)
            {
                Assert.AreEqual("filename is null or empty", ex.Message);
            }
        }

        [TestMethod]
        public void TestDataAccessTextFileMemory()
        {
            IDataAccessObject dao = new DataAccessObjectTextFileMemory();
            IList<Book> books = dao.GetBookByTitle("Android");
            Assert.AreEqual(1, books.Count);
            books = dao.GetBookByTitle("Does not exist");
            Assert.AreEqual(0, books.Count);

            Book book = dao.GetBookByIsbn("1234-5678");
            Assert.IsNull(book);

            book = dao.GetBookByIsbn("978-0-470-74458-1");
            Assert.AreEqual("Sturm", book.Author);

            try
            {
                new DataAccessObjectTextFileMemory(@"c:\doesNotExist.txt");
                Assert.Fail();
            }
            catch (LibraryException) { }

            try
            {
                new DataAccessObjectTextFileMemory("    ");
                Assert.Fail();
            }
            catch (ArgumentException ex)
            {
                Assert.AreEqual("filename is null or empty", ex.Message);
            }
        }

        [TestMethod]
        public void TestDataAccessTextFileDirect()
        {
            IDataAccessObject dao = new DataAccessObjectTextFileDirect();
            IList<Book> books = dao.GetBookByTitle("Android");
            Assert.AreEqual(1, books.Count);
            books = dao.GetBookByTitle("Does not exist");
            Assert.AreEqual(0, books.Count);

            Book book = dao.GetBookByIsbn("1234-5678");
            Assert.IsNull(book);

            book = dao.GetBookByIsbn("978-0-470-74458-1");
            Assert.AreEqual("Sturm", book.Author);

            try
            {
                new DataAccessObjectTextFileDirect(@"c:\doesNotExist.txt");
                Assert.Fail();
            }
            catch (LibraryException) { }

            try
            {
                new DataAccessObjectTextFileDirect("    ");
                Assert.Fail();
            }
            catch (ArgumentException ex)
            {
                Assert.AreEqual("filename is null or empty", ex.Message);
            }
        }
    }
}
